#!/usr/bin/env python
# -*- coding: utf-8 -*-
# -*- Python -*-

"""
 @file ImageFileSelector.py
 @brief Image file selector
 @date $Date$

 @author 株式会社 東日本計算センター

 プロジェクト方針決定後に記載

"""
import sys
import time
sys.path.append(".")

import RTC
import OpenRTM_aist
import filelistwindow_class
import os.path

set_gui = filelistwindow_class.CreateFileSelector()

""" variables definition            """
RTC_INACTIVE = 0
RTC_ACTIVE = 1

""" state notice message definition """
state_messege_text_01 = (
    "[STATE FSELCT] Path information empty! please set the image file.")
state_messege_text_02 = (
    "[STATE FSELCT] Path of the directory or file is wrong. please re-confirm.")
state_messege_text_03 = (
    "[STATE FSELCT] In addition, this program is Japanese does not correspond.")

""" debug message definition        """
debug_messege_text_01 = (
    "[DEBUG FSELCT] onInitialize has been executed")
debug_messege_text_02 = (
    "[DEBUG FSELCT] State change to the onActivated")
debug_messege_text_03 = (
    "[DEBUG FSELCT] State change to the onDeactivated")
debug_messege_text_04 = (
    "[DEBUG FSELCT] ERROR: detected an Aborting state")
debug_messege_text_05 = (
    "[DEBUG FSELCT] onReset has been executed")
debug_messege_text_06 = (
    "[DEBUG FSELCT] Path registration of the input-image file to out port.")
debug_messege_text_07 = (
    "[DEBUG FSELCT] Path registration of the reference-image file to out port.")
debug_messege_text_08 = (
    "[DEBUG FSELCT] Path-> ")

# Import Service implementation class
# <rtc-template block="service_impl">

# </rtc-template>

# Import Service stub modules
# <rtc-template block="consumer_import">
# </rtc-template>


# This module's spesification
# <rtc-template block="module_spec">
imagefileselector_spec = ["implementation_id", "ImageFileSelector",
                          "type_name",         "ImageFileSelector",
                          "description",       "Image file selector",
                          "version",           "1.0",
                          "vendor",            "EAC",
                          "category",          "GUI",
                          "activity_type",     "STATIC",
                          "max_instance",      "1",
                          "language",          "Python",
                          "lang_type",         "SCRIPT",
                          ""]
# </rtc-template>

##
# @class ImageFileSelector
# @brief Image file selector

"""
# 1.ユーザーインターフェースで指定された画像ファイルのフルパス情報を取得する
# 2.ユーザーインターフェースで指定された画像ファイルのフルパス情報をデータポートから出力する
# 
# 出力: 入力画像ファイル/参照画像ファイルのフルパス情報を出力する
# 
# 機能仕様書_ImageFileSelector.docx参照
# 
# なし
# 
"""


class ImageFileSelector(OpenRTM_aist.DataFlowComponentBase):

    ##
    # @brief constructor
    # @param manager Maneger Object
    #
    def __init__(self, manager):
        OpenRTM_aist.DataFlowComponentBase.__init__(self, manager)

        self._d_FilePathInp = RTC.TimedString(RTC.Time(0, 0), "")
        """
        入力画像ファイルのフルパス情報を出力する
         - Type: RTC::TimedString
         - Number: 1
         - Semantics: -
         - Unit: -
         - Frequency: 1秒周期
         - Operation Cycle: 1秒周期
        """
        self._FilePath_InpOut = OpenRTM_aist.OutPort(
            "FilePath_Inp", self._d_FilePathInp)
        self._d_FilePathRef = RTC.TimedString(RTC.Time(0, 0), "")
        """
        参照画像ファイルのフルパス情報を出力する
         - Type: RTC::TimedString
         - Number: 1
         - Semantics: -
         - Unit: -
         - Frequency: 1秒周期
         - Operation Cycle: 1秒周期
        """
        self._FilePath_RefOut = OpenRTM_aist.OutPort(
            "FilePath_Ref", self._d_FilePathRef)

        # initialize of configuration-data.
        # <rtc-template block="init_conf_param">

        # </rtc-template>

        # add to code
        self._act_trig_enable = RTC_INACTIVE

    ##
    # Initialization of the Image file selector RTC
    #
    # The initialize action (on CREATED->ALIVE transition)
    # formaer rtc_init_entry()
    #
    # @return RTC::ReturnCode_t
    #
    # @pre -
    # @post -
    #
    def onInitialize(self):
        # Bind variables and configuration variable

        # Set InPort buffers

        # Set OutPort buffers
        self.addOutPort("FilePath_Inp", self._FilePath_InpOut)
        self.addOutPort("FilePath_Ref", self._FilePath_RefOut)

        # Set service provider to Ports

        # Set service consumers to Ports

        # Set CORBA Service Ports

        # add to code
        print debug_messege_text_01
        self._act_trig_enable = RTC_INACTIVE
        return RTC.RTC_OK

    #    ##
    #    #
    #    # The finalize action (on ALIVE->END transition)
    #    # formaer rtc_exiting_entry()
    #    #
    #    # @return RTC::ReturnCode_t
    #
    #    #
    # def onFinalize(self):
    #
    #    return RTC.RTC_OK

    #    ##
    #    #
    #    # The startup action when ExecutionContext startup
    #    # former rtc_starting_entry()
    #    #
    #    # @param ec_id target ExecutionContext Id
    #    #
    #    # @return RTC::ReturnCode_t
    #    #
    #    #
    # def onStartup(self, ec_id):
    #
    #    return RTC.RTC_OK

    #    ##
    #    #
    #    # The shutdown action when ExecutionContext stop
    #    # former rtc_stopping_entry()
    #    #
    #    # @param ec_id target ExecutionContext Id
    #    #
    #    # @return RTC::ReturnCode_t
    #    #
    #    #
    # def onShutdown(self, ec_id):
    #
    #    return RTC.RTC_OK

        ##
        # Activate the Image file selector RTC
        #
        # The activated action (Active state entry action)
        # former rtc_active_entry()
        #
        # @param ec_id target ExecutionContext Id
        #
        # @return RTC::ReturnCode_t
        #
        # @pre -
        # @post -
        #
    def onActivated(self, ec_id):

        # add to code
        print debug_messege_text_02
        self._act_trig_enable = RTC_ACTIVE
        return RTC.RTC_OK

        ##
        # Deactivate the Image file selector RTC
        #
        # The deactivated action (Active state exit action)
        # former rtc_active_exit()
        #
        # @param ec_id target ExecutionContext Id
        #
        # @return RTC::ReturnCode_t
        #
        # @pre -
        # @post -
        #
    def onDeactivated(self, ec_id):

        # add to code
        print debug_messege_text_03
        self._act_trig_enable = RTC_INACTIVE
        return RTC.RTC_OK

        ##
        # Periodic task of the Image file selector RTC
        #
        # The execution action that is invoked periodically
        # former rtc_active_do()
        #
        # @param ec_id target ExecutionContext Id
        #
        # @return RTC::ReturnCode_t
        #
        # @pre -
        # @post -
        #
    def onExecute(self, ec_id):

        return RTC.RTC_OK

        ##
        # onAborting the Image file selector RTC
        #
        # The aborting action when main logic error occurred.
        # former rtc_aborting_entry()
        #
        # @param ec_id target ExecutionContext Id
        #
        # @return RTC::ReturnCode_t
        #
        # @pre -
        # @post -
        #
    def onAborting(self, ec_id):

        # add to code
        print debug_messege_text_04
        self._act_trig_enable = RTC_INACTIVE

        return RTC.RTC_OK

        ##
        # onError the Image file selector RTC
        #
        # The error action in ERROR state
        # former rtc_error_do()
        #
        # @param ec_id target ExecutionContext Id
        #
        # @return RTC::ReturnCode_t
        #
        # @pre -
        # @post -
        #
    def onError(self, ec_id):

        return RTC.RTC_OK

        ##
        # Reset initialization of the Image file selector RTC
        #
        # The reset action that is invoked resetting
        # This is same but different the former rtc_init_entry()
        #
        # @param ec_id target ExecutionContext Id
        #
        # @return RTC::ReturnCode_t
        #
        # @pre -
        # @post -
        #
    def onReset(self, ec_id):

        # add to code
        print debug_messege_text_05
        return RTC.RTC_OK

    #    ##
    #    #
    #    # The state update action that is invoked after onExecute() action
    #    # no corresponding operation exists in OpenRTm-aist-0.2.0
    #    #
    #    # @param ec_id target ExecutionContext Id
    #    #
    #    # @return RTC::ReturnCode_t
    #    #

    #    #
    # def onStateUpdate(self, ec_id):
    #
    #    return RTC.RTC_OK

    #    ##
    #    #
    #    # The action that is invoked when execution context's rate is changed
    #    # no corresponding operation exists in OpenRTm-aist-0.2.0
    #    #
    #    # @param ec_id target ExecutionContext Id
    #    #
    #    # @return RTC::ReturnCode_t
    #    #
    #    #
    # def onRateChanged(self, ec_id):
    #
    #    return RTC.RTC_OK

    # add to code
    def set_out_inp(self, value):
        if value != "":
            ret_exists = os.path.exists(value)
            if ret_exists == True:
                print debug_messege_text_06

                filepath = value.replace("\\", "/")

                print (debug_messege_text_08 + filepath)
                self._d_FilePathInp.data = str(filepath)
                self._FilePath_InpOut.write()
            else:
                print state_messege_text_02
                print state_messege_text_03
        else:
            print state_messege_text_01

    def set_out_ref(self, value):
        if value != "":
            ret_exists = os.path.exists(value)
            if ret_exists == True:
                print debug_messege_text_07

                filepath = value.replace("\\", "/")

                print (debug_messege_text_08 + filepath)
                self._d_FilePathRef.data = str(filepath)
                self._FilePath_RefOut.write()
            else:
                print state_messege_text_02
                print state_messege_text_03
        else:
            print state_messege_text_01

    def get_act_trig_flag(self):
        return self._act_trig_enable


def ImageFileSelectorInit(manager):
    profile = OpenRTM_aist.Properties(defaults_str=imagefileselector_spec)
    manager.registerFactory(profile,
                            ImageFileSelector,
                            OpenRTM_aist.Delete)


def MyModuleInit(manager):
    ImageFileSelectorInit(manager)

    # Create a component
    comp = manager.createComponent("ImageFileSelector")
    set_gui.master.title("ImageFileSelectorGUI")
    set_gui.set_on_write(comp.set_out_inp, comp.set_out_ref)
    set_gui.get_act_trig(comp.get_act_trig_flag)

    set_gui.build_widget1()
    set_gui.build_widget2()


def main():
    mgr = OpenRTM_aist.Manager.init(sys.argv)
    mgr.setModuleInitProc(MyModuleInit)
    mgr.activateManager()
    mgr.runManager(True)
    set_gui.mainloop()
    mgr.shutdown()

if __name__ == "__main__":
    main()
